SELECT
  id,
  parent_id,
  content,
  location,
  kind,
  creation_date,
  updated_date,
  lastview_date,
  deleted_date

FROM (

  WITH RECURSIVE parents (
    id,
    parent_id,
    content,
    location,
    kind,
    creation_date,
    updated_date,
    lastview_date,
    deleted_date,
    cycle
  )

  AS (

    SELECT
           notes.id,
           note_parents.id,
           notes.content,
		   note_relationships.location,
           notes.kind,
           notes.creation_date,
           notes.updated_date,
           notes.lastview_date,
           notes.deleted_date,
           ','||notes.id||','
      FROM notes
        INNER JOIN note_relationships
          ON notes.id = note_relationships.note_id
          AND notes.kind = 'note'
        INNER JOIN notes as note_parents
          ON note_parents.id = note_relationships.parent_id
      WHERE notes.id
	    IN (SELECT note_id
		      FROM note_kasten_relationships
			    WHERE kasten_id = ?)  -- IMPORTANT: THIS IS THE PARAMETER

    UNION
	SELECT DISTINCT
	       notes.id,
		   next_parent.id,
      	   notes.content,
		   note_relationships.location,
		   notes.kind,
      	   notes.creation_date,
      	   notes.updated_date,
      	   notes.lastview_date,
      	   notes.deleted_date,
      	   parents.cycle||notes.id||','
      FROM notes
      INNER JOIN parents
	    ON parents.parent_id = notes.id
	  LEFT JOIN note_relationships
	    ON note_relationships.note_id = notes.id
	  LEFT JOIN notes as next_parent
	    ON next_parent.id = note_relationships.parent_id
      WHERE parents.cycle NOT LIKE '%,'||notes.id||',%'
  )
  SELECT * from parents);
